import unittest

from imap_tools import MailBox
from tests.utils import get_test_mailbox_config, TEST_MAILBOX_NAME_SET


class ConnectionTest(unittest.TestCase):
    def test_connection(self):
        # simple
        for test_mailbox_name in TEST_MAILBOX_NAME_SET:
            config = get_test_mailbox_config(test_mailbox_name)
            mailbox = MailBox(config['host'])
            self.assertIs(type(mailbox), MailBox)
            login_result = mailbox.login(config['email'], config['password'])
            self.assertEqual(login_result.login_result[0], 'OK')
            logout_result = mailbox.logout()
            self.assertEqual(logout_result[0], 'BYE')
        # with
        for test_mailbox_name in TEST_MAILBOX_NAME_SET:
            config = get_test_mailbox_config(test_mailbox_name)
            with MailBox(config['host']).login(config['email'], config['password']) as mailbox:
                self.assertIs(type(mailbox), MailBox)
                self.assertEqual(mailbox.login_result[0], 'OK')
            # later: test logout result here


if __name__ == "__main__":
    unittest.main()



import unittest

from tests.utils import MailboxTestCase, TEST_MAILBOX_NAME_SET, get_test_mailbox
from imap_tools.errors import MailboxFolderSelectError


class FoldersTest(MailboxTestCase):
    @classmethod
    def setUpClass(cls):
        # delete temp new folders
        for test_mailbox_name in TEST_MAILBOX_NAME_SET:
            mailbox = get_test_mailbox(test_mailbox_name)
            for del_folder in (mailbox.folder_test_new, mailbox.folder_test_new1):
                if mailbox.folder.exists(del_folder):
                    mailbox.folder.delete(del_folder)

    def test_folders(self):
        for mailbox_name, mailbox in self.mailbox_set.items():
            # LIST
            folder_list = mailbox.folder.list(mailbox.folder_test)
            check_folder_set = {mailbox.folder_test_base, mailbox.folder_test_temp1, mailbox.folder_test_temp2}
            self.assertTrue(check_folder_set.issubset(set([i['name'] for i in folder_list])))
            for folder in folder_list:
                self.assertIs(type(folder['delim']), str)
                self.assertIs(type(folder['name']), str)
                for flag in folder['flags']:
                    self.assertIs(type(flag), str)

            # SET, GET
            mailbox.folder.set(mailbox.folder_test_base)
            self.assertEqual(mailbox.folder.get(), mailbox.folder_test_base)
            # CREATE
            mailbox.folder.create(mailbox.folder_test_new)
            folder_list_names = [i['name'] for i in mailbox.folder.list(mailbox.folder_test)]
            self.assertTrue(mailbox.folder_test_new in folder_list_names)
            # EXISTS
            self.assertTrue(mailbox.folder.exists(mailbox.folder_test_new))
            # RENAME
            mailbox.folder.rename(mailbox.folder_test_new, mailbox.folder_test_new1)
            folder_list_names = [i['name'] for i in mailbox.folder.list(mailbox.folder_test)]
            self.assertTrue(mailbox.folder_test_new1 in folder_list_names)
            self.assertFalse(mailbox.folder_test_new in folder_list_names)
            # DELETE
            mailbox.folder.delete(mailbox.folder_test_new1)
            folder_list_names = [i['name'] for i in mailbox.folder.list(mailbox.folder_test)]
            self.assertFalse(mailbox.folder_test_new1 in folder_list_names)
            # STATUS
            for status_key, status_val in mailbox.folder.status(mailbox.folder_test_base).items():
                self.assertIs(type(status_key), str)
                self.assertIs(type(status_val), int)
            # error
            with self.assertRaises(MailboxFolderSelectError):
                mailbox.folder.set('__nonexistent_folder__')


if __name__ == "__main__":
    unittest.main()



import os
import unittest
import configparser
from imap_tools import MailBox

TEST_MAILBOX_NAME_SET = {'YANDEX', 'ZIMBRA', 'MAIL_RU', 'OUTLOOK', 'GOOGLE'}


def get_test_mailbox_config(mailbox_name: str) -> dict:
    config = configparser.ConfigParser()
    for config_path in (r'../tests/credentials.ini', r'tests/credentials.ini'):
        if not os.path.exists(config_path):
            continue
        config.read(config_path)
        return dict(
            host=config[mailbox_name]['host'],
            email=config[mailbox_name]['email'],
            password=config[mailbox_name]['password'],
            path_separator=config[mailbox_name]['path_separator'],
            test_folder=config[mailbox_name]['test_folder'],
        )


def get_test_mailbox(mailbox_name: str):
    # get config
    config = get_test_mailbox_config(mailbox_name)

    # add test attributes to MailBox
    class MailBoxTestEx(MailBox):
        def __init__(self, *args):
            super().__init__(*args)
            test_folder = config['test_folder']
            path_separator = config['path_separator']
            self.folder_test = '{}'.format(test_folder)
            self.folder_test_base = '{}{}base'.format(test_folder, path_separator)
            self.folder_test_temp1 = '{}{}temp1'.format(test_folder, path_separator)
            self.folder_test_temp2 = '{}{}temp2'.format(test_folder, path_separator)
            self.folder_test_new = '{}{}new'.format(test_folder, path_separator)
            self.folder_test_new1 = '{}{}new1'.format(test_folder, path_separator)

    # create mailbox instance
    mailbox = MailBoxTestEx(config['host'])
    # connect
    mailbox.login(config['email'], config['password'])
    # done
    return mailbox


class MailboxTestCase(unittest.TestCase):
    def setUp(self):
        self.mailbox_set = dict()
        for test_mailbox_name in TEST_MAILBOX_NAME_SET:
            self.mailbox_set[test_mailbox_name] = get_test_mailbox(test_mailbox_name)

    def tearDown(self):
        for mailbox in self.mailbox_set.values():
            mailbox.logout()



import re
import inspect
import datetime
from email.utils import getaddresses, parsedate_to_datetime
from email.header import decode_header, Header

SHORT_MONTH_NAMES = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')


def cleaned_uid_set(uid_set: str or [str] or iter) -> str:
    """
    Prepare set of uid for use in IMAP commands
    uid_set may be:
        str, that is comma separated uids
        Iterable, that contains str uids
        Generator with "fetch" name, implicitly gets all uids
    """
    # str
    if type(uid_set) is str:
        if re.search(r'^(\d+,)*\d+$', uid_set):  # *optimization for already good str
            return uid_set
        uid_set = uid_set.split(',')
    # Generator
    if inspect.isgenerator(uid_set) and getattr(uid_set, '__name__', None) == 'fetch':
        uid_set = tuple(msg.uid for msg in uid_set if msg.uid)
    # Iterable
    try:
        uid_set_iter = iter(uid_set)
    except TypeError:
        raise TypeError('Wrong uid_set arg type: "{}"'.format(type(uid_set)))
    # check uid types
    for uid in uid_set_iter:
        if type(uid) is not str:
            raise TypeError('uid "{}" is not string'.format(str(uid)))
        if not uid.strip().isdigit():
            raise TypeError('Wrong uid: "{}"'.format(uid))
    return ','.join((i.strip() for i in uid_set))


def check_command_status(command_result: tuple, exception: type, expected='OK'):
    """
    Check that IMAP command responses status equals <expected> status
    If not, raise specified <exception>
    :param command_result: imap command result
    :param exception: exception subclass of UnexpectedCommandStatusError, that raises
    :param expected: expected command status
    """
    typ, data = command_result[0], command_result[1]
    if typ != expected:
        raise exception(command_result=command_result, expected=expected)


def decode_value(value: bytes or str, encoding=None) -> str:
    """Converts value to utf-8 encoding"""
    if isinstance(encoding, str):
        encoding = encoding.lower()
    if isinstance(value, bytes):
        try:
            return value.decode(encoding or 'utf-8', 'ignore')
        except LookupError:  # unknown encoding
            return value.decode('utf-8', 'ignore')
    return value


def parse_email_addresses(raw_header: str or Header) -> (dict,):
    """
    Parse email addresses from header
    :param raw_header: example: '=?UTF-8?B?0J7Qu9C1=?= <name@company.ru>,\r\n "\'\\"z, z\\"\'" <imap.tools@ya.ru>'
    :return: tuple(dict(name: str, email: str, full: str))
    """
    result = []
    if type(raw_header) is Header:
        raw_header = decode_value(*decode_header(raw_header)[0])
    for raw_name, email in getaddresses([raw_header]):
        name = decode_value(*decode_header(raw_name)[0]).strip()
        email = email.strip()
        if not (name or email):
            continue
        result.append({
            'email': email if '@' in email else '',
            'name': name,
            'full': '{} <{}>'.format(name, email) if name and email else name or email
        })
    return tuple(result)


def parse_email_date(value: str) -> datetime.datetime:
    """
    Parsing the date described in rfc2822
    1900-1-1 for unparsed, may be naive or with tzinfo
    """
    try:
        return parsedate_to_datetime(value)
    except Exception:  # noqa
        pass
    match = re.search(r'(?P<date>\d{1,2}\s+(' + '|'.join(SHORT_MONTH_NAMES) + r')\s+\d{4})\s+' +
                      r'(?P<time>\d{1,2}:\d{1,2}(:\d{1,2})?)\s*' +
                      r'(?P<zone_sign>[+-])?(?P<zone>\d{4})?', value)
    if match:
        group = match.groupdict()
        day, month, year = group['date'].split()
        time_values = group['time'].split(':')
        zone_sign = int('{}1'.format(group.get('zone_sign') or '+'))
        zone = group['zone']
        return datetime.datetime(
            year=int(year),
            month=SHORT_MONTH_NAMES.index(month) + 1,
            day=int(day),
            hour=int(time_values[0]),
            minute=int(time_values[1]),
            second=int(time_values[2]) if len(time_values) > 2 else 0,
            tzinfo=datetime.timezone(datetime.timedelta(
                hours=int(zone[:2]) * zone_sign,
                minutes=int(zone[2:]) * zone_sign
            )) if zone else None,
        )
    else:
        return datetime.datetime(1900, 1, 1)


def quote(value: str or bytes):
    if isinstance(value, str):
        return '"' + value.replace('\\', '\\\\').replace('"', '\\"') + '"'
    else:
        return b'"' + value.replace(b'\\', b'\\\\').replace(b'"', b'\\"') + b'"'


def pairs_to_dict(items: list) -> dict:
    """Example: ['MESSAGES', '3', 'UIDNEXT', '4'] -> {'MESSAGES': '3', 'UIDNEXT': '4'}"""
    if len(items) % 2 != 0:
        raise ValueError('An even-length array is expected')
    return dict((items[i * 2], items[i * 2 + 1]) for i in range(len(items) // 2))



import unittest
import datetime

from imap_tools import utils
from imap_tools.errors import ImapToolsError, UnexpectedCommandStatusError, MailboxCopyError


class UtilsTest(unittest.TestCase):

    def test_cleaned_uid_set(self):
        # *cleaned_uid_set tested enough in test_query.py
        pass

    def test_quote(self):
        self.assertEqual(utils.quote('str привет'), '"str привет"')
        self.assertEqual(utils.quote('str \\'), '"str \\\\"')
        self.assertEqual(utils.quote(b'\xd1\x8f'), b'"\xd1\x8f"')
        self.assertEqual(utils.quote(b'\\ \xd1\x8f  \\'), b'"\\\\ \xd1\x8f  \\\\"')

    def test_pairs_to_dict(self):
        self.assertEqual(utils.pairs_to_dict(['MESSAGES', '3', 'UIDNEXT', '4']), {'MESSAGES': '3', 'UIDNEXT': '4'})
        with self.assertRaises(ValueError):
            utils.pairs_to_dict(['1', '2', '3'])

    def test_decode_value(self):
        self.assertEqual(utils.decode_value('str привет 你好', 'not matter'), 'str привет 你好')
        self.assertEqual(utils.decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0'), 'str да 你')
        self.assertEqual(utils.decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0', 'utf8'), 'str да 你')
        self.assertEqual(utils.decode_value(b'\xef\xf0\xe8\xe2\xe5\xf2', 'cp1251'), 'привет')
        self.assertEqual(utils.decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0', 'wat?'), 'str да 你')

    def test_check_command_status(self):
        self.assertIsNone(utils.check_command_status(('EXP', 'command_result_data'), MailboxCopyError, expected='EXP'))
        self.assertIsNone(utils.check_command_status(('OK', 'res'), UnexpectedCommandStatusError))
        with self.assertRaises(TypeError):
            self.assertFalse(utils.check_command_status(('NOT_OK', 'test'), ImapToolsError))
        with self.assertRaises(MailboxCopyError):
            self.assertFalse(utils.check_command_status(('BYE', ''), MailboxCopyError, expected='OK'))

    def test_parse_email_date(self):
        for val, exp in (
                ('1 Jan 2000 00:00', datetime.datetime(2000, 1, 1, 0, 0)),
                ('1 Feb 2000 00:00', datetime.datetime(2000, 2, 1, 0, 0)),
                ('1 Mar 2000 00:00', datetime.datetime(2000, 3, 1, 0, 0)),
                ('1 Apr 2000 00:00', datetime.datetime(2000, 4, 1, 0, 0)),
                ('1 May 2000 00:00', datetime.datetime(2000, 5, 1, 0, 0)),
                ('1 Jun 2000 00:00', datetime.datetime(2000, 6, 1, 0, 0)),
                ('1 Jul 2000 00:00', datetime.datetime(2000, 7, 1, 0, 0)),
                ('1 Aug 2000 00:00', datetime.datetime(2000, 8, 1, 0, 0)),
                ('1 Sep 2000 00:00', datetime.datetime(2000, 9, 1, 0, 0)),
                ('1 Oct 2000 00:00', datetime.datetime(2000, 10, 1, 0, 0)),
                ('1 Nov 2000 00:00', datetime.datetime(2000, 11, 1, 0, 0)),
                ('1 Dec 2000 00:00', datetime.datetime(2000, 12, 1, 0, 0)),

                ('=) wat 7 Jun 2017 09:23!',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('7 Jun 2017 09:23',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('Wed, 7 Jun 2017 09:23',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('Wed, 7 Jun 2017 09:23:14',
                 datetime.datetime(2017, 6, 7, 9, 23, 14)),
                ('Wed, 7 Jun 2017 09:23:14 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, 14, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23:14 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, 14, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 -2359',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone(datetime.timedelta(-1, 60)))),
                ('7 Jun 2017 09:23 +0530 (UTC) asd',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone(datetime.timedelta(0, 19800)))),

                ('7 Bad 2017 09:23', datetime.datetime(1900, 1, 1, 0, 0)),
        ):
            self.assertEqual(utils.parse_email_date(val), exp)

    def test_parse_email_addresses(self):
        self.assertEqual(
            utils.parse_email_addresses('=?UTF-8?B?0J7Qu9C1=?= <name@company.ru>,\r\n "\'\\"z, z\\"\'" <ya@ya.ru>\f'),
            ({'email': 'name@company.ru', 'name': 'Оле', 'full': 'Оле <name@company.ru>'},
             {'email': 'ya@ya.ru', 'name': '\'"z, z"\'', 'full': '\'"z, z"\' <ya@ya.ru>'}))
        self.assertEqual(
            utils.parse_email_addresses(' <ivan@mail.ru>'),
            ({'email': 'ivan@mail.ru', 'name': '', 'full': 'ivan@mail.ru'},))
        self.assertEqual(
            utils.parse_email_addresses('ivan'),
            ({'email': '', 'name': '', 'full': 'ivan'},))  # *в этом случае ivan считается email-ом
        self.assertEqual(
            utils.parse_email_addresses('你好 <chan@mail.ru>'),
            ({'email': 'chan@mail.ru', 'name': '你好', 'full': '你好 <chan@mail.ru>'},))
        self.assertEqual(
            utils.parse_email_addresses(' "hi" <bad_mail.wow> '),
            ({'email': '', 'name': 'hi', 'full': 'hi <bad_mail.wow>'},))
        self.assertEqual(
            utils.parse_email_addresses('=?utf-8?Q?ATO.RU?= <subscriptions@ato.ru>'),
            ({'email': 'subscriptions@ato.ru', 'name': 'ATO.RU', 'full': 'ATO.RU <subscriptions@ato.ru>'},))



import unittest

from imap_tools import imap_utf7


class ImapUtf7Test(unittest.TestCase):
    data = (
        ('Test', b'Test'),
        ('Test One more', b'Test One more'),
        ('Might & Magic', b'Might &- Magic'),
        ('Might & magic', b'Might &- magic'),
        ('Imap&\xffworld', b'Imap&-&AP8-world'),
        ('\xff\xfe\xfd\xfc', b'&AP8A,gD9APw-'),
        ('\x00', b'&AAA-'),
        ('hello, Jackie Chan 你好，成龙', b'hello, Jackie Chan &T2BZff8MYhCfmQ-'),  # RFC-2060
        ('str \t\n\r\f\vwhitespace \t\n\r\f\v', b'str &AAkACgANAAwACw-whitespace &AAkACgANAAwACw-')
    )

    def test_encode(self):
        for string, code in self.data:
            self.assertEqual(imap_utf7.encode(string), code)

    def test_decode(self):
        for string, code in self.data:
            self.assertEqual(string, imap_utf7.decode(code))

    def test_printable_chars(self):
        for code in range(32, 127):
            if code == 38:  # &
                continue
            self.assertEqual(bytes((code,)), imap_utf7.encode(chr(code)))
            self.assertEqual(chr(code), imap_utf7.decode(bytes((code,))))
        self.assertEqual(imap_utf7.encode('&'), b'&-')
        self.assertEqual(imap_utf7.decode(b'&-'), '&')



from .query import Q, AND, OR, NOT, Header, A, O, N, H
from .mailbox import BaseMailBox, MailBox, MailBoxUnencrypted
from .message import MailMessage, Attachment, MailMessageFlags
from .folder import MailBoxFolderManager, MailBoxFolderStatusOptions
from .errors import *

__version__ = '0.22.0'



import unittest
import datetime as dt

from imap_tools.query import ParamConverter, A, AND, OR, NOT, H


class QueryTest(unittest.TestCase):

    def test_cleaners(self):
        def fetch():
            class Msg:
                uid = '1'

            yield Msg()

        def not_fetch():
            yield 1

        for cleaned_fn_name, good_vals, bad_vals in (
                ('cleaned_bool', (True, False), (1, 'str', [], {}, type, b'1')),
                ('cleaned_date', (dt.date.today(),), (dt.datetime.now(), 1, 's', [], {}, type, True, b'1')),
                ('cleaned_uint', (0, 1, 145), (-1, 'str', [], {}, type, True, b'1')),
                ('cleaned_str', ('', 'good', 'я 你好'), (1, [], {}, type, True, b'1')),
                ('cleaned_true', (True,), (1, 'str', [], {}, type, False, b'1')),
                ('cleaned_uid', ('1', '1,2', ['1', '2'], [], {}, fetch()), (1, type, True, b'1', '', not_fetch())),
                ('cleaned_header', (H('X-Google-Smtp', '123'), H('a', '1')), (1, 's', ['s', 1], {}, type, False, b'1')),
        ):
            cleaned_fn = getattr(ParamConverter, cleaned_fn_name)
            for good in good_vals:
                self.assertIsNotNone(cleaned_fn('key_does_not_matter', good))
            for bad in bad_vals:
                with self.assertRaises(TypeError):
                    cleaned_fn('key_does_not_matter', bad)

    def test_converters(self):
        self.assertEqual(A(answered=True), '(ANSWERED)')
        self.assertEqual(A(answered=False), '(UNANSWERED)')
        self.assertEqual(A(seen=True), '(SEEN)')
        self.assertEqual(A(seen=False), '(UNSEEN)')
        self.assertEqual(A(flagged=True), '(FLAGGED)')
        self.assertEqual(A(flagged=False), '(UNFLAGGED)')
        self.assertEqual(A(draft=True), '(DRAFT)')
        self.assertEqual(A(draft=False), '(UNDRAFT)')
        self.assertEqual(A(deleted=True), '(DELETED)')
        self.assertEqual(A(deleted=False), '(UNDELETED)')
        self.assertEqual(A(keyword='KEY1'), '(KEYWORD KEY1)')
        self.assertEqual(A(no_keyword='KEY2'), '(UNKEYWORD KEY2)')

        self.assertEqual(A(from_='from@ya.ru'), '(FROM "from@ya.ru")')
        self.assertEqual(A(to='to@ya.ru'), '(TO "to@ya.ru")')
        self.assertEqual(A(subject='hello'), '(SUBJECT "hello")')
        self.assertEqual(A(body='body text'), '(BODY "body text")')
        self.assertEqual(A(body='hi'), '(BODY "hi")')
        self.assertEqual(A(text='"quoted text"'), '(TEXT "\\"quoted text\\"")')
        self.assertEqual(A(text='hi'), '(TEXT "hi")')
        self.assertEqual(A(bcc='bcc@ya.ru'), '(BCC "bcc@ya.ru")')
        self.assertEqual(A(cc='cc@ya.ru'), '(CC "cc@ya.ru")')

        self.assertEqual(A(date=dt.date(2000, 3, 15)), '(ON 15-Mar-2000)')
        self.assertEqual(A(date_gte=dt.date(2000, 3, 15)), '(SINCE 15-Mar-2000)')
        self.assertEqual(A(date_lt=dt.date(2000, 3, 15)), '(BEFORE 15-Mar-2000)')
        self.assertEqual(A(sent_date=dt.date(2000, 3, 15)), '(SENTON 15-Mar-2000)')
        self.assertEqual(A(sent_date_gte=dt.date(2000, 3, 15)), '(SENTSINCE 15-Mar-2000)')
        self.assertEqual(A(sent_date_lt=dt.date(2000, 3, 15)), '(SENTBEFORE 15-Mar-2000)')

        self.assertEqual(A(size_gt=1024), '(LARGER 1024)')
        self.assertEqual(A(size_lt=512), '(SMALLER 512)')

        self.assertEqual(A(new=True), '(NEW)')
        self.assertEqual(A(old=True), '(OLD)')
        self.assertEqual(A(recent=True), '(RECENT)')
        self.assertEqual(A(all=True), '(ALL)')

        self.assertEqual(A(header=H('X-Google-Smtp-Source', '123')), '(HEADER "X-Google-Smtp-Source" "123")')
        self.assertEqual(A(uid='1,2'), '(UID 1,2)')
        self.assertEqual(A(uid=['3', '4']), '(UID 3,4)')

        self.assertEqual(A(gmail_label="TestLabel"), '(X-GM-LABELS "TestLabel")')

    def test_format_date(self):
        self.assertEqual(ParamConverter.format_date(dt.date(2000, 1, 15)), '15-Jan-2000')
        self.assertEqual(ParamConverter.format_date(dt.date(2000, 12, 15)), '15-Dec-2000')

    def test_logic_operators(self):
        self.assertEqual(AND(text='hello', new=True), '(TEXT "hello" NEW)')
        self.assertEqual(OR(text='hello', new=True), '(OR TEXT "hello" NEW)')
        self.assertEqual(NOT(text='hello', new=True), 'NOT (TEXT "hello" NEW)')
        self.assertEqual(A(AND(to='one@mail.ru'), AND(to='two@mail.ru')), '((TO "one@mail.ru") (TO "two@mail.ru"))')
        self.assertEqual(
            OR(date=[dt.date(2019, 10, 1), dt.date(2019, 10, 10), dt.date(2019, 10, 15), dt.date(2019, 10, 20)]),
            '(OR OR OR ON 1-Oct-2019 ON 10-Oct-2019 ON 15-Oct-2019 ON 20-Oct-2019)')
        self.assertEqual(
            A(OR(from_='from@ya.ru', text='"the text"'), NOT(OR(A(answered=False), A(new=True))), to='to@ya.ru'),
            '((OR FROM "from@ya.ru" TEXT "\\"the text\\"") NOT ((OR (UNANSWERED) (NEW))) TO "to@ya.ru")')

    def test_header(self):
        header = H('key1', 'val1')
        self.assertEqual(header.name, '"key1"')
        self.assertEqual(header.value, '"val1"')
        with self.assertRaises(TypeError):
            str(H('key1', eval('1')))
        with self.assertRaises(TypeError):
            str(H(eval('1'), 'val1'))



class ImapToolsError(Exception):
    """Base lib error"""


class MailboxFolderStatusValueError(ImapToolsError):
    """Wrong folder status value error"""


class UnexpectedCommandStatusError(ImapToolsError):
    """Unexpected status in IMAP command response"""

    def __init__(self, command_result: tuple, expected: str):
        """
        :param command_result: imap command result
        :param expected: expected command status
        """
        self.command_result = command_result
        self.expected = expected

    def __str__(self):
        return 'Response status "{exp}" expected, but "{typ}" received. Data: {data}'.format(
            exp=self.expected, typ=self.command_result[0], data=str(self.command_result[1]))


class MailboxFolderSelectError(UnexpectedCommandStatusError):
    pass


class MailboxFolderCreateError(UnexpectedCommandStatusError):
    pass


class MailboxFolderRenameError(UnexpectedCommandStatusError):
    pass


class MailboxFolderDeleteError(UnexpectedCommandStatusError):
    pass


class MailboxFolderStatusError(UnexpectedCommandStatusError):
    pass


class MailboxLoginError(UnexpectedCommandStatusError):
    pass


class MailboxLogoutError(UnexpectedCommandStatusError):
    pass


class MailboxSearchError(UnexpectedCommandStatusError):
    pass


class MailboxFetchError(UnexpectedCommandStatusError):
    pass


class MailboxExpungeError(UnexpectedCommandStatusError):
    pass


class MailboxDeleteError(UnexpectedCommandStatusError):
    pass


class MailboxCopyError(UnexpectedCommandStatusError):
    pass


class MailboxFlagError(UnexpectedCommandStatusError):
    pass
